﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

using AssetList = System.Collections.Generic.List<BlockDefense.Classes.Asset>;
using SpriteList = System.Collections.Generic.List<BlockDefense.Classes.Sprite>;
using TextureLibrary = System.Collections.Generic.Dictionary<string, Microsoft.Xna.Framework.Graphics.Texture2D>;
using TypeLibrary = System.Collections.Generic.Dictionary<string, System.Type>;

namespace BlockDefense.Classes
{
    public enum GameState { Menu, Instructions, InGame, GameOver };
    public enum MenuState { Start, Instructions, Quit };
    public enum GameOverState { Retry, Menu, Quit };

    public class GameLibrary
    {
        public  AssetList       Assets      { get; private set; }
        public  bool            Quit        { get; private set; }
        public  Defender        Defender    { get; private set; }
        public  SpriteList      Asteroids   { get; private set; }
        public  SpriteList      Shields     { get; private set; }
        public  SpriteList      Static      { get; private set; }
        public  TextureLibrary  Textures    { get; private set; }

        private bool            Initialized { get; set; }
        private float           Density     { get; set; }
        private float           Rate        { get; set; }
        private GameState       State       { get; set; }
        private GameOverState   GoState     { get; set; }
        private int             Count       { get; set; }
        private int             Level       { get; set; }
        private int             Multiplier  { get; set; }
        private int             Power       { get; set; }
        private int             Score       { get; set; }
        private int             Time        { get; set; }
        private int             PowerTime   { get; set; }
        private MenuState       MenuState   { get; set; }
        private Random          Rand        { get; set; }
        private SpriteFont      Font        { get; set; }
        

        public GameLibrary()
        {
            Assets = new AssetList();
            Asteroids = new SpriteList();
           
            Rand = new Random();
            Shields = new SpriteList();
            Static = new SpriteList();
            Textures = new TextureLibrary();

            Count = 0;
            Density = 1.05f;
            Initialized = false;
            Level = 1;
            Multiplier = 1;
            Score = 0;
            Rate = .01f;
            Time = 200;
            Power = 0;
            PowerTime = 2000;
            State = GameState.Menu;
        }
        public void Initialize()
        {
            _LoadAssetList();
        }
        public void LoadContent(ContentManager content)
        {
            foreach (Asset a in Assets)
            {
                if (a.Type == "Texture2D")
                    Textures.Add(a.Name, content.Load<Texture2D>(a.Path));
                else if(a.Type == "SpriteFont")
                    Font = content.Load<SpriteFont>(a.Path);
            }

            Static.Add(new Sprite(Textures["Background"], new Vector2(0, 0), 1));
            Static.Add(new Sprite(Textures["Border"], new Vector2(0, 30), 1));
            Static.Add(new Sprite(Textures["Border"], new Vector2(0, 565), 1));
        }
        public void Update(GameTime gameTime, Rectangle clientBounds)
        {
            SpriteList remove = new SpriteList();
            Rectangle overlap;

            switch (State)
            {
                case GameState.Menu:
                    Time += gameTime.ElapsedGameTime.Milliseconds;
                    if (GamePad.GetState(PlayerIndex.One).Buttons.A == ButtonState.Pressed && Time > 200)
                    {
                        switch (MenuState)
                        {
                            case MenuState.Start:
                                Time = 0;
                                _SetInitialState(clientBounds);
                                State = GameState.InGame;
                                break;
                            case MenuState.Instructions:
                                Time = 0;
                                State = GameState.Instructions;
                                break;
                            case MenuState.Quit:
                                Quit = true;
                                break;
                        }
                    }
                    else if (GamePad.GetState(PlayerIndex.One).ThumbSticks.Left.Y > 0 && Time > 200)
                    {
                        Time = 0;
                        switch(MenuState)
                        {
                            case MenuState.Start:
                                break;
                            case MenuState.Instructions:
                                MenuState = MenuState.Start;
                                break;
                            case MenuState.Quit:
                                MenuState = MenuState.Instructions;
                                break;
                        }
                    }
                    else if (GamePad.GetState(PlayerIndex.One).ThumbSticks.Left.Y < 0 && Time > 200)
                    {
                        Time = 0;
                        switch (MenuState)
                        {
                            case MenuState.Start:
                                MenuState = MenuState.Instructions;
                                break;
                            case MenuState.Instructions:
                                MenuState = MenuState.Quit;
                                break;
                            case MenuState.Quit:
                                break;
                        }
                    }
                    break;
                case GameState.Instructions:
                    Time += gameTime.ElapsedGameTime.Milliseconds;
                    if (GamePad.GetState(PlayerIndex.One).Buttons.A == ButtonState.Pressed && Time > 200)
                    {
                        Time = 0;
                        State = GameState.Menu;
                    }
                    break;
                case GameState.InGame:

                    if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                        State = GameState.GameOver;

                    Time += gameTime.ElapsedGameTime.Milliseconds;
                    PowerTime += gameTime.ElapsedGameTime.Milliseconds;

                    if (Time > 1000 * Rate)
                    {
                        Time = 0;
                        Asteroids.Add(new Asteroid(Textures["Asteroid"], new Vector2(clientBounds.Width + 1, Rand.Next(clientBounds.Height - 74 - Textures["Asteroid"].Height) + 37), -MathHelper.Clamp((float)Rand.NextDouble() * 4f, 2f, 4f)));
                        Count++;
                        if (Count > 15)
                        {
                            Count = 0;
                            Rate = Rate / Density;
                            Level++;
                        }
                    }

                    if (PowerTime > 2000 && GamePad.GetState(PlayerIndex.One).Triggers.Right > 0)
                    {
                        Time = 0;
                        Power -= 1;
                        if (Power < 0)
                            Power = 0;
                    }

                    foreach (Asteroid a in Asteroids)
                    {
                        if (a.OutOfBounds)
                        {
                            if (Math.Sign(a.Speed) == -1)
                            {
                                Multiplier -= 1;
                                if (Multiplier < 1)
                                    Multiplier = 1;
                            }
                            remove.Add(a);
                        }

                        else
                        {
                            overlap = Rectangle.Intersect(Defender.Bounds, a.Bounds);
                            if (Math.Sign(a.Speed) == -1 && (overlap.Width != 0 || overlap.Height != 0))
                            {
                                a.Speed = -a.Speed;
                                Score += Multiplier * 10;
                                Power += 2;
                                if (Power > 100)
                                    Power = 100;
                            }

                            var items = (Asteroids.FindAll(
                                delegate(Sprite r)
                                {
                                    bool removed = remove.Exists(delegate(Sprite t) { return t.Id == r.Id || t.Id == a.Id; });
                                    bool collision = Math.Sign(a.Speed / ((Asteroid)r).Speed) == -1 || (Math.Sign(a.Speed) == 1 && Math.Sign(((Asteroid)r).Speed) == 1);
                                    overlap = Rectangle.Intersect(r.Bounds, a.Bounds);
                                    return r.Id != a.Id && collision && !removed && (overlap.Width != 0 || overlap.Height != 0);
                                }));

                            if (items.Count != 0)
                            {
                                Score += Multiplier * 20;
                                Multiplier++;
                                remove.Add(a);
                                remove.AddRange(items);
                                Power += 4;
                                if (Power > 100)
                                    Power = 100;
                            }

                            var item = Shields.Find(
                                delegate(Sprite r)
                                {
                                    overlap = Rectangle.Intersect(r.Bounds, a.Bounds);
                                    return overlap.Width != 0 || overlap.Height != 0;
                                });

                            if (item != null)
                            {
                                remove.Add(item);
                                remove.Add(a);
                                if (Multiplier / 2 == 0)
                                    Multiplier = 1;
                                else
                                    Multiplier /= 2;
                            }
                        }
                    }

                    foreach (Sprite s in remove)
                    {
                        Asteroids.Remove(s);
                        Shields.Remove(s);
                    }
                    remove.Clear();

                    Defender.Update(gameTime, clientBounds);
                    foreach (Asteroid a in Asteroids)
                    {
                        if (GamePad.GetState(PlayerIndex.One).Triggers.Right > 0 && Math.Sign(a.Speed) == -1 && Power > 0)
                            a.Slowed = true;
                        else
                            a.Slowed = false;
                        a.Update(gameTime, clientBounds);
                    }
                    if (Shields.Count == 0)
                    {
                        State = GameState.GameOver;
                        Time = 200;
                    }
                    break;
                case GameState.GameOver:
                    Time += gameTime.ElapsedGameTime.Milliseconds;
                    if (GamePad.GetState(PlayerIndex.One).Buttons.A == ButtonState.Pressed && Time > 200)
                    {
                        switch (GoState)
                        {
                            case GameOverState.Retry:
                                Time = 0;
                                _SetInitialState(clientBounds);
                                State = GameState.InGame;
                                break;
                            case GameOverState.Menu:
                                Time = 1000;
                                State = GameState.Menu;
                                break;
                            case GameOverState.Quit:
                                Quit = true;
                                break;
                        }
                    }
                    else if (GamePad.GetState(PlayerIndex.One).ThumbSticks.Left.Y > 0 && Time > 200)
                    {
                        Time = 0;
                        switch (GoState)
                        {
                            case GameOverState.Retry:
                                break;
                            case GameOverState.Menu:
                                GoState = GameOverState.Retry;
                                break;
                            case GameOverState.Quit:
                                GoState = GameOverState.Menu;
                                break;
                        }
                    }
                    else if (GamePad.GetState(PlayerIndex.One).ThumbSticks.Left.Y < 0 && Time > 200)
                    {
                        Time = 0;
                        switch (GoState)
                        {
                            case GameOverState.Retry:
                                GoState = GameOverState.Menu;
                                break;
                            case GameOverState.Menu:
                                GoState = GameOverState.Quit;
                                break;
                            case GameOverState.Quit:
                                break;
                        }
                    }
                    break;
            }
        }
        public void Draw(GameTime gameTime, SpriteBatch spriteBatch)
        {
            foreach (Sprite s in Static)
                s.Draw(gameTime, spriteBatch);

            switch (State)
            {
                case GameState.Menu:
                    spriteBatch.Draw(Textures["Title"], new Vector2(0, 0), Color.White);
                    switch (MenuState)
                    {
                        case MenuState.Start:
                            spriteBatch.DrawString(Font, "Start", new Vector2(375, 300), Color.Red);
                            spriteBatch.DrawString(Font, "Instructions", new Vector2(345, 330), Color.White);
                            spriteBatch.DrawString(Font, "Quit", new Vector2(378, 360), Color.White);
                            break;
                        case MenuState.Instructions:
                            spriteBatch.DrawString(Font, "Start", new Vector2(375, 300), Color.White);
                            spriteBatch.DrawString(Font, "Instructions", new Vector2(345, 330), Color.Red);
                            spriteBatch.DrawString(Font, "Quit", new Vector2(378, 360), Color.White);
                            break;
                        case MenuState.Quit:
                            spriteBatch.DrawString(Font, "Start", new Vector2(375, 300), Color.White);
                            spriteBatch.DrawString(Font, "Instructions", new Vector2(345, 330), Color.White);
                            spriteBatch.DrawString(Font, "Quit", new Vector2(378, 360), Color.Red);
                            break;
                    }
                    break;
                case GameState.Instructions:
                    spriteBatch.Draw(Textures["Instructions"], new Vector2(0, 0), Color.White);
                    break;
                case GameState.InGame:
                    Defender.Draw(gameTime, spriteBatch);

                    foreach (Sprite s in Asteroids)
                        s.Draw(gameTime, spriteBatch);

                    foreach (Sprite s in Shields)
                        s.Draw(gameTime, spriteBatch);

                    spriteBatch.Draw(Textures["Power"], new Vector2(0, 0), new Rectangle(0, 0, Power * 8, 30), Color.White);
                    spriteBatch.DrawString(Font, "Level: " + Level.ToString() + " Multiplier: " + Multiplier.ToString() + "x Score: " + Score.ToString(), new Vector2(5, 575), Color.White);
                    spriteBatch.DrawString(Font, "Temporal Displacment Drive Power", new Vector2(5, 5), Color.White);
                    break;
                case GameState.GameOver:
                    spriteBatch.DrawString(Font, "GAME OVER", new Vector2(348, 200), Color.White);
                    spriteBatch.DrawString(Font, "Score : " + Score.ToString(), new Vector2(348, 230), Color.White);
                    switch (GoState)
                    {
                        case GameOverState.Retry:
                            spriteBatch.DrawString(Font, "Retry", new Vector2(375, 300), Color.Red);
                            spriteBatch.DrawString(Font, "Menu", new Vector2(374, 330), Color.White);
                            spriteBatch.DrawString(Font, "Quit", new Vector2(378, 360), Color.White);
                            break;
                        case GameOverState.Menu:
                            spriteBatch.DrawString(Font, "Retry", new Vector2(375, 300), Color.White);
                            spriteBatch.DrawString(Font, "Menu", new Vector2(374, 330), Color.Red);
                            spriteBatch.DrawString(Font, "Quit", new Vector2(378, 360), Color.White);
                            break;
                        case GameOverState.Quit:
                            spriteBatch.DrawString(Font, "Retry", new Vector2(375, 300), Color.White);
                            spriteBatch.DrawString(Font, "Menu", new Vector2(374, 330), Color.White);
                            spriteBatch.DrawString(Font, "Quit", new Vector2(378, 360), Color.Red);
                            break;
                    }
                    break;
            }

        }  
        private void _SetInitialState(Rectangle clientBounds)
        {
            int i;
            int center = clientBounds.Height / 2;
            Time = 0;
            Defender = new Defender(Textures["Defender"], new Vector2(140, center - Textures["Defender"].Height / 2), 0);

            for (i = 0; i < 4; i++)
            {
                Shields.Add(new Sprite(Textures["Base"], new Vector2(0, center - Textures["Base"].Height * (i + 1) - (2*i + 1)), 0));
                Shields.Add(new Sprite(Textures["RedShield"], new Vector2(64, center - Textures["Base"].Height * (i + 1) - (2 * i + 1)), 0));
                Shields.Add(new Sprite(Textures["YellowShield"], new Vector2(82, center - Textures["Base"].Height * (i + 1) - (2 * i + 1)), 0));
                Shields.Add(new Sprite(Textures["GreenShield"], new Vector2(100, center - Textures["Base"].Height * (i + 1) - (2 * i + 1)), 0));

                Shields.Add(new Sprite(Textures["Base"], new Vector2(0, center + Textures["Base"].Height * i + (2*i + 1)), 0));
                Shields.Add(new Sprite(Textures["RedShield"], new Vector2(64, center + Textures["Base"].Height * i + (2 * i + 1)), 0));
                Shields.Add(new Sprite(Textures["YellowShield"], new Vector2(82, center + Textures["Base"].Height * i + (2 * i + 1)), 0));
                Shields.Add(new Sprite(Textures["GreenShield"], new Vector2(100, center + Textures["Base"].Height * i + (2 * i + 1)), 0));
            }

            Asteroids.Clear();
            Count = 0;
            Level = 0;
            Score = 0;
            Multiplier = 1;
            Rate = 1;
        }
        private void _LoadAssetList()
        {
            Assets.Add(new Asset("Asteroid", @"Images/Asteroid", "Texture2D"));
            Assets.Add(new Asset("Background", @"Images/Background", "Texture2D"));
            Assets.Add(new Asset("Base", @"Images/Base", "Texture2D"));
            Assets.Add(new Asset("Border", @"Images/Border", "Texture2D"));
            Assets.Add(new Asset("Defender", @"Images/Defender", "Texture2D"));
            Assets.Add(new Asset("GreenShield", @"Images/GreenShield", "Texture2D"));
            Assets.Add(new Asset("YellowShield", @"Images/YellowShield", "Texture2D"));
            Assets.Add(new Asset("RedShield", @"Images/RedShield", "Texture2D"));
            Assets.Add(new Asset("Power", @"Images/Power", "Texture2D"));
            Assets.Add(new Asset("Title", @"Images/Title", "Texture2D"));
            Assets.Add(new Asset("Instructions", @"Images/Instructions", "Texture2D"));
            Assets.Add(new Asset("Arial", @"Fonts/Arial", "SpriteFont"));
        }
  }
}
